   .486
   .model flat, stdcall
   option casemap :none

   include \masm32\include\windows.inc
   include \masm32\include\user32.inc
   include \masm32\include\kernel32.inc
   include \masm32\include\masm32.inc
   include \masm32\macros\macros.asm
   includelib \masm32\lib\masm32.lib
   includelib \masm32\lib\user32.lib
   includelib \masm32\lib\kernel32.lib

   crc32 PROTO :DWORD,:DWORD  
   CRC32 PROTO :DWORD,:DWORD
   
.data
   CRCtable dd 256 DUP (?)
   buff     db 30 dup(0)
.code
start:

   clock = 1
   call main
      
   print SADD(13,10,"Press Enter to exit...",13,10)    
   invoke StdIn,addr buff,1
   
   exit
   
; 
; This is a heavily modified version of the main procedure
; from Hutch's SCAN7.ASM.
; 

main proc

   LOCAL fname :DWORD
   LOCAL pMem  :DWORD
   LOCAL tc    :DWORD
   LOCAL fsize :DWORD
   LOCAL buffer[260]:BYTE
   mov fname, ptr$(buffer)    ; get pointer to buffer
   invoke GetCL,1, fname      ; get 1st command line argument

   invoke exist,fname         ; check if the command line file exists
   cmp eax, 0
   je not_exist               ; display error if not

   mov pMem, InputFile(fname) ; load the file into memory

   invoke filesize,fname
   mov fsize,eax

   IF clock
      invoke GetTickCount
      mov tc, eax
   ENDIF
   REPEAT 20     
      invoke crc32,pMem,fsize
   ENDM
   push  eax
   IF clock
      invoke GetTickCount
      sub eax, tc
      mov edx,eax
      invoke dw2a,edx,addr buffer
      print SADD("crc32 ms (20x):")
      print addr buffer
      print SADD(13,10)
   ENDIF
   pop   edx
   invoke dw2hex,edx,addr buffer
   print SADD("crc32 return value:")
   print addr buffer
   print SADD(13,10)

   call InitCRC32Table   
      
   IF clock
      invoke GetTickCount
      mov tc, eax
   ENDIF
   REPEAT 20
      invoke CRC32,pMem,fsize
   ENDM
   push  eax
   IF clock
      invoke GetTickCount
      sub eax, tc
      mov edx,eax
      invoke dw2a,edx,addr buffer
      print SADD("CRC32 ms (20x):")
      print addr buffer
      print SADD(13,10)
   ENDIF
   pop   edx
   invoke dw2hex,edx,addr buffer
   print SADD("CRC32 return value:")
   print addr buffer
   print SADD(13,10)  

   jmp pexit

   not_exist:

      print chr$("Cannot find that file",13,10)
      print chr$("SYNTAX: CRC-32 filename.ext",13,10)

   pexit:

      free pMem            ; free the source memory
      
   ret
      
main endp

; 
; This is my (MichaelW) bit-by-bit CRC-32 procedure.
; 

align 4

crc32 proc uses esi pMem:DWORD,memlen:DWORD

   ; These correct for THIS implementation and CRC-32.
   poly     EQU 04C11DB7h
   init     EQU 46AF6449h
   xorout   EQU 0FFFFFFFFh

   mov   esi,pMem
   mov   ecx,memlen

   ; Use EDX for the working register.
   mov   edx,init

   byteLoop:
      ; Load the next byte into AH.
      mov   ah,[esi]
      ; Reflect AH into AL.
      REPEAT 8
         rcl   ah,1
         rcr   al,1
      ENDM
      ; Start of unrolled bit loop.
      REPEAT 8
         ; Shift next bit into CF.
         shl   al,1
         ; Shift CF into working register.
         rcl   edx,1
         ; Shifted-out bit now in CF, if was 0 then skip XOR.
         jnc   @F
         xor   edx,poly
      @@:
      ENDM
      inc   esi
      dec   ecx
   jnz   byteLoop

   ; This conversion for non-augmented byte streams only.
   REPEAT 32
      shl   edx,1
      jnc   @F
      xor   edx,poly
   @@:
   ENDM

   ; Reflect bits in EDX into EAX.
   REPEAT 32
      rcl   edx,1
      rcr   eax,1
   ENDM

   ; Do final XOR.
   xor   eax,xorout

   ; Return CRC value in EAX.
   ret

crc32 endp

; 

convert proc

   ; This procedure shows how the "non-direct" initial register
   ; value is derived from the standard initial register value.

   mov   eax,0FFFFFFFFh
   mov   ecx,32

   bitLoop:
      mov   edx,1
      and   edx,eax
      jz    @F
      xor   eax,poly
   @@:
      shr   eax,1
      or    edx,edx
      jz    @f
      or    eax,80000000h
   @@:
      dec   ecx
   jnz   bitLoop

   ret

convert endp

; 
; These are MASM versions of Donkey's GoAsm CRC-32 procedures,
; adapted from routines by Thomas Bleeker (MadWizard),table-
; driven and *much* faster than my bit-by-bit procedure.
; 

InitCRC32Table proc uses ebx esi edi
   mov edi, offset CRCtable  - 4 
   xor ecx, ecx 
    
   M1: 
   mov eax, ecx 
   mov ebx, 0EDB88320h 
   add edi, 4 
   mov esi, 8 
   @@: 
      shr eax, 1    
      sbb edx, edx  
      and edx, ebx 
      xor eax, edx 
   dec esi 
   jnz @B 
   inc ecx 
   mov [edi], eax 
   cmp ecx, 256 
   jb  M1 

   mov eax,offset CRCtable 
   ret
InitCRC32Table endp

align 4

CRC32 proc uses ebx esi edi lpBuffer, cbBuffer 

   mov esi, lpBuffer 
   mov edi, offset CRCtable 
   mov edx, cbBuffer
    
   shr edx, 1 
   or ecx, -1 
   xor eax, eax 

   @@: 
   mov al, [esi] 
   xor al, cl 
   shr ecx, 8 
   mov ebx, [edi+4*eax] 
   xor ecx, ebx 

   mov al, [esi+1] 
   xor al, cl 
   shr ecx, 8 
   mov ebx, [edi+4*eax] 
   add esi,2 
   xor ecx, ebx 

   dec edx 
   jnz @B 

   test dword ptr[cbBuffer], 1 
   jz @F 
   mov al, [esi] 
   xor al, cl 
   inc esi 
   shr ecx, 8 
   mov ebx, [edi+4*eax] 
   xor ecx, ebx 
   @@: 

   mov eax, ecx    
   not eax 
   ret
      
CRC32 endp

; 

end start

